/*      CaS, performs conversion from textfiles to .CAS files
 *
 * (c) Copyright 1996 Tom Wheeley <tomw@tsys.demon.co.uk> All Rights Reserved
 *
 * You may freely use parts of this source code, provided that you:
 *  + Acknowledge the fact you used CaS source code in your program, visibly
 *    to your program's users.
 *  + Send the author of CaS a copy of your program.
 *  + You are not under obligation to include source for your program, but
 *    that is preferred.
 *
 */
 
/*
 * CAS.H  standard includes and defines.  should be included by all .c files
 */

#ifndef CAS_H
#define CAS_H

#define CAS

/* standard #includes */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h> 
#include <string.h>
#include <stdarg.h>

#define CAS_MAJOR_VER                    "1.0"
#define CAS_MINOR_VER                    "0"

#undef CAS_DOS
#undef CAS_UNIX

#ifdef __linux__
# define CAS_VER_OS                      "Linux"
# define CAS_DIRSEP                      '/'
# define CAS_COMM			 "termios"
# define CAS_UNIX
#else
# ifdef __DJGPP__
#  define CAS_VER_OS                     "msdos-dpmi"
#  define CAS_DIRSEP                     '/'
#  define CAS_COMM                       "SVAsync"
#  define CAS_DOS
#  define CAS_UNIX		/* djgpp is getting closer all the time! */
# else
#  ifdef __TURBOC__
#   define CAS_VER_OS                    "msdos-real"
#   define CAS_DIRSEP                    '\\'
#   define CAS_COMM                      "Async"
#   define CAS_DOS
#  else
#   define CAS_VER_OS                    "Exotic"
#   define CAS_DIRSEP                    '/'
#   undef CAS_COMM
#  endif
# endif
#endif

#ifdef NO_COMM          /* outside define to disable communications */
# undef CAS_COMM
#endif

#define CAS_VER_NUM         CAS_MAJOR_VER "." CAS_MINOR_VER
#define CAS_VER_TEMP         CAS_VER_OS " version " CAS_VER_NUM
#ifdef CAS_COMM
# define CAS_VER_STR    CAS_VER_TEMP
#else
# define CAS_VER_STR    CAS_VER_TEMP " (no-comms)"
#endif

typedef int bool;
#define TRUE 				1
#define FALSE				0

#define EXIT_CMD            1   /* Error in Command Line */
#define EXIT_FILE           2   /* File access error */
#define EXIT_MEM            3   /* Unable to allocate memory */
#define EXIT_COMM           4   /* Communications error */
#define EXIT_FORMAT         5   /* Error in file format */
#define EXIT_BUG            99  /* Bug / unimplemented feature */

#define MDL_UNKNOWN              0xff
#define MDL_7700                 0x02
#define MDL_9700                 0x04
#define MDL_9800                 0x08
#define MDL_9850                 0x10
#define MDL_9950                 MDL_9850

#define SUBF_UNKNOWN        0   /* Unknown type, try next subfiletype */
#define SUBF_OK             1   /* OK */
#define SUBF_UNSUPP         2   /* recognised, but unsupported.  try raw */

#define SUBF_9700		'9'
#define SUBF_9X50		'5'
#define SUBF_RAW		'R'

/*
 * All data structures in cas.h refer to the abstract forms of data stored,
 * and so only correspond with file formats where the storage methods
 * coincide.  File-format specific structures are in the relevant fXXX.h
 * (or sXXXX.h for .CAS / comms structures/values)
 */

/* Flags structure */

typedef enum {fUnknown,ftxt,fCAS,fFXP,fBMP,fGIF,fCOM} formats;

typedef struct {
  char *argv0;
  struct {
    bool set;
    formats format;
    char *tokfile;
    char **argv;
  } list;
  struct {
    formats format;
    char *filename;
    char *tokfile;
    int subf;
    char **argv;
  } output;
  struct {
    formats format;
    char *filename;
    char *tokfile;
    int subf;
    char **argv;
  } input;
  int model;
  int conv;
  bool verbose;
  bool pager;
  FILE *debug;
} flagtype;

/* Sizes of data structures */

#define CAS_MEMSIZE             65025   /* Size of buffer to store data */

/* Information structure */

typedef enum {tUnknown,tSingleProg,tMultiProg,tSingleEditor,tMultiEditor,
        tOneFMem,tSixFMem,tGraphs,tStdDev,tLinReg,tSSMono,tSSColour,
        tAll,tDynaGraph,tBackup,tPolyEq,tSimEq,tRecTab,tFuncTab,tGrZoom,
        tGrRange,tDefMem,tAlphaMem,tVarMem,tMultiMat,tSingleMat,tEnd,tRaw
        } types;

typedef struct {
  types type;
  void *data;           /* Pointer to beginning of data */
  void *dataend;        /* Pointer to mark end of data, for protection */
  size_t length;        /* Actual length of data */
  int numblks;          /* Number of header/data pairs (fcas/fcom) */
  unsigned int size1;   /* Reserved for filetype specific info */
  unsigned int size2;
  void *aux;            /* controlling info for the data */
} casinfo;

/*
 *  Random data stored in casinfo:
 *
 *  tSingleProg
 *    size1 = Number of programs = 1
 *    size2 = Size of program + 2
 *    aux   = Pointer to prginfo structure
 *
 *  tMultiProg
 *    size1 = Number of programs = 38
 *    size2 = Size of all programs + 2
 *    aux   = Pointer to array of 38 prginfo structures
 *
 *  tSingleEditor
 *    size1 = Number of editor files =1 
 *    size2 = 
 *    aux   = Pointer to prginfo structure
 *
 *  tMultiEditor
 *    size1 = Number of editor files
 *    size2 = 
 *    aux   = Pointer to array of n prginfo structures
 *
 *  tScreenshot
 *    size1 = Width of screen
 *    size2 = Height of screen
 *    aux   = Pointer to an int.  1 -> 9700 style, 2 -> 9800 style
 */


/* other structures */

typedef struct {
  unsigned char *data;
  int size;
  int progtype;         /* undefined for editor files */
  unsigned char filename[13];    /* undefined for programs */
  unsigned char password[13];    /* undefined for programs */
} prginfo;

/* As CaS stores the program type in the same way to the .CAS file, these
   defines are here and not in c9700.h */

#define CAS_PT_SSTO             0x02
#define CAS_PT_MATRIX           0x04
#define CAS_PT_SD               0x10
#define CAS_PT_LR               0x20
#define CAS_PT_BASEN            0x40
#define CAS_PT_SDRAW            0x80

/* screenshot defines */

#define SS_XMAX     255
#define SS_YMAX     255

#endif /* CAS_H */

/* end of cas.h */
