/*      CaS, performs conversion from textfiles to .CAS files
 *
 * (c) 1996 Tom Wheeley <tomw@tsys.demon.co.uk>  All Rights Reserved
 *
 * You may freely use parts of this source code, provided that you:
 *  + Acknowledge the fact you used CaS source code in your program, visibly
 *    to your program's users.
 *  + Send the author of CaS a copy of your program.
 *  + You are not under obligation to include source for your program, but
 *    that is preferred.
 *
 */
 
#include "cas.h"
#include "misc.h"
#include "svasync.h"
#include "commsys.h"

#define COMM_TIMEOUT          500000L     /* 500,000 */

extern flagtype flags;

int COMsysInit(char *filename, int baud, char parity, int handshake)
{
  int comport=-1;
  int parnum=EVEN_PARITY;
  int hand=0;

  if (!stricmp(filename,"COM1")) comport=COM1;
  if (!stricmp(filename,"COM2")) comport=COM2;
  if (!stricmp(filename,"COM3")) comport=COM3;
  if (!stricmp(filename,"COM4")) comport=COM4;
  if (comport==-1) return FALSE;

  switch (parity) {
    case CASPORT_NONE: parnum=NO_PARITY;break;
    case CASPORT_ODD: parnum=ODD_PARITY;break;
    case CASPORT_EVEN: ;
    default: parnum=EVEN_PARITY;break;
  }

  if (handshake&CASPORT_DTR) hand |=DTR;
  if (handshake&CASPORT_RTS) hand |=RTS;

  /* Initialise COM port */

  if (SVAsyncInit(comport))
    return FALSE;

  atexit(SVAsyncStop);  /* cleans up COM port upon exit */

  /* Initialise the FIFO buffer on the UART */

  switch (SVAsyncFifoInit()) {
    case 0: Msg(M_VERBOSE,"comm","16550 FIFO or better detected");break;
    case 1: Msg(M_VERBOSE,"comm","16450 (or 16550 no FIFO) detected");break;
    case 2: Msg(M_VERBOSE,"comm","8250 or below detected");break;
    default: Msg(M_VERBOSE,"comm","Unknown UART detected");
  }

  /* set baud rate and parity.  Calc uses 8 data bits + 2 stop bits */

  SVAsyncSet(baud,BITS_8|parnum|STOP_2);

  /* raise power lines */

  if (hand) SVAsyncHand(hand);

  return TRUE;
}

void COMsysClose(void)
{
  SVAsyncStop();
}

void COMsysPutChar(unsigned char data)
{
  SVAsyncOut(data);
}

unsigned char COMsysGetChar(void)
{
  for(;;) {
    if (SVAsyncInStat())
      return SVAsyncIn();
  }
  /* Msg(EXIT_COMM,"comm/read","Timeout reading from COM port"); */
  return 0;
}

unsigned char COMsysGetCharTimeout(int *success_r,unsigned int modifier)
{
  unsigned long x;
  unsigned long timeout;

  timeout=COMM_TIMEOUT * 100 / modifier;

  for(x=0;x<timeout;x++) {
    if (SVAsyncInStat()) {
      *success_r=TRUE;
      return SVAsyncIn();
    }
  }
  *success_r=FALSE;
  return 1;
}


/* end of cdjgpp.c */
