/*      CaS, performs conversion from textfiles to .CAS files
 *
 * (c) 1996 Tom Wheeley <tomw@tsys.demon.co.uk>  All Rights Reserved
 *
 * You may freely use parts of this source code, provided that you:
 *  + Acknowledge the fact you used CaS source code in your program, visibly
 *    to your program's users.
 *  + Send the author of CaS a copy of your program.
 *  + You are not under obligation to include source for your program, but
 *    that is preferred.
 *
 */
 
#include "cas.h"
#include "misc.h"
#include "args.h"
#include "token.h"
#include "list.h"
#include "fbmp.h"

extern flagtype flags;
extern unsigned char pow2[8];

void BMPReadData(char *filename, casinfo *info_r)
{
  FILE *bmp=NULL;
  unsigned char (*data)[SS_XMAX][SS_YMAX];
  int x,y,z;
  unsigned char chunk;
  int flg_inv;

  if (!(bmp=tsys_fopen(filename,"rb",FALSE))) {
    Msg(EXIT_FILE,"bmp/read","Unable to open `%s'",filename);
    return;
  }
  
  if (!BMPReadHeader(bmp,info_r)) {
    Msg(M_DEBUG,"bmp/open","Error reading header");
    tsys_fclose(bmp);
    return;
  }

  flg_inv=ArgsIsSet("inverse",flags.input.argv);
  data = info_r->data;
  for (y=0;y<info_r->size2;y++)
    for (x=0;x<info_r->size1;x+=8) {
      fread(&chunk,1,1,bmp);
      for (z=0;z<8;z++)
        (*data)[x+z][y]=((chunk & pow2[7-z])!=0)?!flg_inv:flg_inv;
    }

  tsys_fclose(bmp);
}


void BMPWriteData(char *filename,casinfo *info)
{
  FILE *bmp=NULL;
  int x,y,z;
  unsigned char (*data)[SS_XMAX][SS_YMAX];
  unsigned char chunk;
  int flg_inv;

  if (info->type!=tSSMono) {
    Msg(EXIT_BUG,"cas/write","Unsupported file variation: %s",ListType(info->type));
    return;
  }

  flg_inv=ArgsIsSet("inverse",flags.output.argv);

  if (!(bmp=tsys_fopen(filename,"wb",FALSE))) return;

  BMPWriteHeader(bmp,info);

  data=info->data;

  for (y=0;y<info->size2;y++)
    for (x=0;x<info->size1;x+=8) {
      chunk=0;
      for (z=0;z<8;z++)
        chunk+=(( (*data)[x+z][y]?!flg_inv:flg_inv)?1:0) * pow2[7-z];
      fwrite(&chunk,1,1,bmp);
    }

  tsys_fclose(bmp);
}


void BMPWriteHeader(FILE *bmp, casinfo *info)
{
  A_BITMAPFILEHEADER bmfh;
  A_BITMAPINFOHEADER bmih;
  RGBQUAD col[1];

  bmfh.bfType='B'+('M' << 8);
  bmfh.bfReserved1=0;bmfh.bfReserved2=0;
  bmfh.bfOffBits = sizeof(bmfh)
                 + sizeof(BITMAPINFOHEADER)
                 + sizeof(RGBQUAD) * 2;
  bmfh.bfSize=bmfh.bfOffBits + info->size1*info->size2/8;

  bmih.biSize=sizeof(BITMAPINFOHEADER);
  bmih.biWidth=info->size1;
  bmih.biHeight=info->size2;
  bmih.biPlanes=1;
  bmih.biBitCount=1;
  bmih.biCompression=0;bmih.biSizeImage=0;
  bmih.biXPelsPerMeter=0x00000b13;  /* taken from BMP made by CasioWin */
  bmih.biYPelsPerMeter=0x00000b13;
  bmih.biClrUsed=0;
  bmih.biClrImportant=0;

  col[0].rgbBlue=0xff;col[0].rgbGreen=0xff;
  col[0].rgbRed=0xff;col[0].rgbReserved=0;

  col[1].rgbBlue=0;col[1].rgbGreen=0;
  col[1].rgbRed=0;col[1].rgbReserved=0;

  fwrite(&bmfh,1,sizeof(bmfh),bmp);
  fwrite(&bmih.biSize,1,sizeof(BITMAPINFOHEADER),bmp);
  fwrite(&col[0],1,sizeof(RGBQUAD),bmp);
  fwrite(&col[1],1,sizeof(RGBQUAD),bmp);

  Msg(M_DEBUG,"bmp/writehdr","Data offset: %04x",ftell(bmp));
}


bool BMPReadHeader(FILE *bmp,casinfo *info_r)
{
  A_BITMAPFILEHEADER bmfh;
  A_BITMAPINFOHEADER bmih;
  UINT btLE='B'+('M'<<8);
  UINT btBE=('B'<<8)+'M';

  fread(&bmfh,1,sizeof(bmfh),bmp);
  fread(&bmih.biSize,1,sizeof(BITMAPINFOHEADER),bmp);
  fseek(bmp,bmfh.bfOffBits,SEEK_SET);
  
  info_r->type=tSSMono;
  info_r->size1=bmih.biWidth;           /* Width */
  info_r->size2=bmih.biHeight;          /* Height */
  info_r->length=bmih.biWidth * bmih.biHeight;
  info_r->numblks=1;

  Msg(M_DEBUG,"bmp/readhdr","height: %ld",bmih.biHeight);
  Msg(M_DEBUG,"bmp/readhdr","width : %ld",bmih.biWidth);

  if (bmfh.bfType!=btLE && bmfh.bfType!=btBE)
    Msg(EXIT_FILE,"bmp/readhdr","Not a Bitmap file");
  if (bmih.biHeight>SS_YMAX)
    Msg(EXIT_FILE,"bmp/readhdr","Bitmap too high %ld > %d",bmih.biHeight,SS_YMAX);
  if (bmih.biWidth>SS_XMAX)
    Msg(EXIT_FILE,"bmp/readhdr","Bitmap too wide %ld > %d",bmih.biWidth,SS_XMAX);
  if (bmih.biBitCount!=1)
    Msg(EXIT_BUG,"bmp/readhdr","Unsupported file variation: Colour Screenshot");

  return TRUE;
}


/* end of fbmp.c */
