/*      CaS, performs conversion from textfiles to .CAS files
 *
 * (c) 1996 Tom Wheeley <tomw@tsys.demon.co.uk>  All Rights Reserved
 *
 * You may freely use parts of this source code, provided that you:
 *  + Acknowledge the fact you used CaS source code in your program, visibly
 *    to your program's users.
 *  + Send the author of CaS a copy of your program.
 *  + You are not under obligation to include source for your program, but
 *    that is preferred.
 *
 */
 
#include "cas.h"
#include "misc.h"
#include "args.h"
#include "s9700.h"
#include "sraw.h"
#include "fcas.h"

extern flagtype flags;
extern unsigned char pow2[8];

static FILE *cas=NULL;

void CASReadData(char *filename, casinfo *info_r)
{
  flags.input.subf=SUBF_9700;

  if (vArgsIsSet(flags.input.argv,3,"9850","9950","9x50"))
    flags.output.subf=SUBF_9X50;
  if (ArgsIsSet("raw",flags.input.argv)) flags.input.subf=SUBF_RAW;

  if (!(cas=tsys_fopen(filename,"rb",FALSE))) {
    Msg(EXIT_FILE,"cas/read","Unable to open `%s'",filename);
    return;
  }

  switch(flags.input.subf) {
    case SUBF_9700: ReadData_9700(info_r, CASReadBlock);break;
    /* case SUBF_9X50: ReadData_9X50(info_r, CASReadBlock);break; */
    case SUBF_RAW: CASReadRaw(info_r);break;
    default: Msg(EXIT_BUG,"cas/read","Unknown subfiletype `%c'",flags.input.subf);
  }

  tsys_fclose(cas);
}


void CASWriteData(char *filename,casinfo *info)
{
  if (!(cas=tsys_fopen(filename,"wb",FALSE))) {
    Msg(EXIT_FILE,"cas/read","Unable to open `%s'",filename);
    return;
  }

  /* default CAS subfile type.  Should be more sophisticated, eg unsupported
  types should be raw etc.  Also output should be same as input by default */

  flags.output.subf=SUBF_9700;
  if (flags.input.format==fCAS||flags.input.format==fCOM)
    flags.output.subf=flags.input.subf;
  if (info->type==tUnknown||info->type==tRaw)
    flags.output.subf=SUBF_RAW;

  if (vArgsIsSet(flags.input.argv,3,"9850","9950","9x50"))
    flags.output.subf=SUBF_9X50;
  if (ArgsIsSet("raw",flags.output.argv)) flags.output.subf=SUBF_RAW;

  switch(flags.output.subf) {
    case SUBF_9700: WriteData_9700(info, CASWriteBlock);break;
    /* case SUBF_9x50: WriteData_9x50(info, CASWriteBlock);break; */
    case SUBF_RAW:  CASWriteRaw(info);break;
    default: Msg(EXIT_BUG,"cas/write","Unknown subfiletype `%c'",flags.output.subf);
  }

  tsys_fclose(cas);
}


void CASReadBlock(unsigned char *data,size_t size)
{
  char colon;
  unsigned char rcsum, ccsum;

  if ((colon=getc(cas))!=':')
    Msg(EXIT_FORMAT,"cas/read","Expected `:', read `%c'",colon);    
  if (fread(data,1,size,cas)!=size)
    Msg(EXIT_FORMAT,"cas/read","Unable to read 0x%04lx bytes.",size);
  rcsum=getc(cas);
  ccsum=CalcCSum(data,size);
  if (rcsum!=ccsum)
    Msg(M_WARN,"cas/csum","Checksum error.  read 0x%02x, expected 0x%02x",
                rcsum,ccsum);
}


void CASWriteBlock(unsigned char *data,size_t size)
{
  unsigned char ccsum;

  Msg(M_DEBUG,"cas/wrblk","Writing %04lx bytes",size);
  putc(':',cas);
  if (fwrite(data,1,size,cas)!=size)
    Msg(EXIT_FORMAT,"cas/read","Unable to write 0x%04lx bytes.",size);
  ccsum=CalcCSum(data,size);
  putc(ccsum,cas);
}


void CASReadRaw(casinfo *info_r)
{
  long size;

  fseek(cas,0,SEEK_END);size=ftell(cas);rewind(cas);
  Msg(M_DEBUG,"cas/raw","Data size: %d bytes",size);
  if (size>CAS_MEMSIZE)
    Msg(EXIT_FILE,"cas/read","File too large: %ld",size);
  fread(info_r->data,1,size,cas);

  info_r->type=tRaw;
  info_r->length=size;
  info_r->numblks=1;
}


void CASWriteRaw(casinfo *info)
{
  if (flags.input.subf!=SUBF_RAW)
    Msg(EXIT_CMD,"cas/wr-raw","Cannot convert data to raw subfiletype");

  fwrite(info->data,1,info->length,cas);
}


/* end of fcas.c */
