/*      CaS, performs conversion from textfiles to .CAS files
 *
 * (c) 1996 Tom Wheeley <tomw@tsys.demon.co.uk>  All Rights Reserved
 *
 * You may freely use parts of this source code, provided that you:
 *  + Acknowledge the fact you used CaS source code in your program, visibly
 *    to your program's users.
 *  + Send the author of CaS a copy of your program.
 *  + You are not under obligation to include source for your program, but
 *    that is preferred.
 *
 */
 
#include "cas.h"
#include "misc.h"
#include "args.h"
#include "s9700.h"
#include "fcom.h"
#include "commsys.h"

extern flagtype flags;

#ifdef CAS_COMM /* covers most of this file */ 

void COMReadData(char *filename, casinfo *info_r)
{
  int baud,parity,hand;
  unsigned char rdchar;
  
  flags.input.subf=SUBF_9700;
  if (vArgsIsSet(flags.input.argv,3,"9850","9950","9x50"))
    flags.input.subf=SUBF_9X50;
  if (ArgsIsSet("raw",flags.input.argv)) flags.input.subf=SUBF_RAW;

  COMParseArgs(flags.input.argv, &baud, &parity, &hand);

  Msg(M_VERBOSE,"comm","Settings: %s, %u, 8, %c, 2%s%s",filename,baud,parity,
    hand&CASPORT_DTR?", DTR":"",hand&CASPORT_RTS?", RTS":"");

  if (!(COMsysInit(filename,baud,parity,hand)))
    Msg(EXIT_COMM,"comm","Unable to initialise communications");

  /* Wait for calculator */

  do {
    rdchar=COMsysGetChar();
  } while (rdchar!=CASPORT_TX_READY);
  COMsysPutChar(CASPORT_RX_READY);

  Msg(M_DEBUG,"comm","Handshake successful");

  /* Receive data */

  switch (flags.input.subf) {
    case SUBF_9700: ReadData_9700(info_r, COMGetBlock);break;
  /*  case SUBF_9x50: ReadData_9x50(info_r, COMGetBlock);break; */
    case SUBF_RAW: COMReadRaw(info_r);break;
  }
  
  COMsysClose();
}


void COMWriteData(char *filename,casinfo *info)
{
  int baud,parity,hand;

  /* default CAS subfile type.  Should be more sophisticated, eg unsupported
  types should be raw etc.  Also output should be same as input by default */

  flags.output.subf=SUBF_9700;
  if (flags.input.format==fCAS||flags.input.format==fCOM)
    flags.output.subf=flags.input.subf;
  if (info->type==tUnknown||info->type==tRaw)
    flags.output.subf=SUBF_RAW;

  if (vArgsIsSet(flags.input.argv,3,"9850","9950","9x50"))
    flags.output.subf=SUBF_9X50;
  if (ArgsIsSet("raw",flags.input.argv)) flags.output.subf=SUBF_RAW;

  COMParseArgs(flags.output.argv, &baud, &parity, &hand);

  Msg(M_VERBOSE,"comm","Settings: %s, %u, 8, %c, 2%s%s",filename,baud,parity,
    hand&CASPORT_DTR?", DTR":"",hand&CASPORT_RTS?", RTS":"");

  if (ArgsIsSet("pause",flags.output.argv)) {
    Msg(M_MSG,"comm","press <enter> to begin communications");
    Pager(PAGER_FORCE|PAGER_NOW|PAGER_NOMSG);
  }

  /* Initialise COM port */

  if (!(COMsysInit(filename,baud,parity,hand)))
    Msg(EXIT_COMM,"comm","Unable to initialise communications");

  /* Handshake with calculator */

  do {
    COMsysPutChar(CASPORT_TX_READY);
  } while (COMsysGetChar() != CASPORT_RX_READY);
  Msg(M_DEBUG,"comm","Handshake successful");

  /* send data */

  switch (flags.output.subf) {
    case SUBF_9700: WriteData_9700(info, COMPutBlock);break;
  /*  case SUBF_9x50: WriteData_9x50(info, COMPutBlock);break; */
    case SUBF_RAW: COMWriteRaw(info);break;
  }

  COMsysClose();
}


void COMParseArgs(char **argv, int *baud, int *parity, int *hand)
{
  char *tmpstr;

  /* defaults */
  (*parity)=CASPORT_EVEN;
  (*baud)=9600;
  (*hand)=CASPORT_DTR;

  if ((tmpstr=ArgsGetString("parity",argv))) {
    if (!stricmp(tmpstr,"even")) (*parity)=CASPORT_EVEN;
    if (!stricmp(tmpstr,"none")) (*parity)=CASPORT_NONE;
    if (!stricmp(tmpstr,"odd")) (*parity)=CASPORT_ODD;
  }
  if (ArgsIsSet("even",argv)) (*parity)=CASPORT_EVEN;
  if (ArgsIsSet("none",argv)) (*parity)=CASPORT_NONE;
  if (ArgsIsSet("odd",argv)) (*parity)=CASPORT_ODD;
  if (ArgsIsSet("E",argv)) (*parity)=CASPORT_EVEN;
  if (ArgsIsSet("N",argv)) (*parity)=CASPORT_NONE;
  if (ArgsIsSet("O",argv)) (*parity)=CASPORT_ODD;

  if ((tmpstr=ArgsGetString("(*baud)",argv))) {
    if (!strcmp(tmpstr,"9600")) (*baud)=9600;
    if (!strcmp(tmpstr,"4800")) (*baud)=4800;
    if (!strcmp(tmpstr,"2400")) (*baud)=2400;
    if (!strcmp(tmpstr,"1200")) (*baud)=1200;
  }
  if (ArgsIsSet("9600",argv)) (*baud)=9600;
  if (ArgsIsSet("4800",argv)) (*baud)=4800;
  if (ArgsIsSet("2400",argv)) (*baud)=2400;
  if (ArgsIsSet("1200",argv)) (*baud)=1200;

  if (ArgsIsSet("RTS",argv))
    (*hand) |= CASPORT_RTS;
  if (ArgsIsSet("noDTR",argv))
    (*hand) &= ~CASPORT_DTR;

}


void COMGetBlock(unsigned char *data,size_t size)
{
  unsigned char rcsum, ccsum;
  unsigned char rdchar;
  unsigned char *curpos=NULL;
  long count;

  do {
    if ((rdchar=COMsysGetChar())!=':')
      Msg(M_WARN,"cas/read","Expected `:', read 0x%02x",rdchar);
  } while (rdchar!=':');

  ccsum=0;curpos=data;
  for (count=0;count<size;count++) {
    rdchar=COMsysGetChar();
    *curpos++=rdchar;
  };

  rcsum=COMsysGetChar();
  ccsum=CalcCSum(data,size);
  if (rcsum!=ccsum)
    Msg(M_WARN,"comm/csum","Checksum error.  read 0x%02x, expected 0x%02x",
                rcsum,ccsum);

  COMsysPutChar(CASPORT_OK);
}


void COMPutBlock(unsigned char *data, size_t size)
{
  unsigned char *curpos=NULL;
  unsigned char ccsum;
  unsigned char rdchar;
  long count;

  curpos=data;

  COMsysPutChar(':');
  for(count=0;count<size;count++) {
    COMsysPutChar(*curpos++);
  }
  ccsum=CalcCSum(data,size);
  COMsysPutChar(ccsum);

  rdchar=COMsysGetChar();
  COMCheckResponse(rdchar);
}

void COMCheckResponse(unsigned char response)
{
  unsigned char rdchar='\xff';

  switch (response) {
    case CASPORT_OK:
             break;
    case CASPORT_BADCSUM:
             Msg(EXIT_COMM,"comm","Checksum error");
    case CASPORT_AREAFULL:
             if (ArgsIsSet("timid",flags.output.argv)) {
               COMsysPutChar(CASPORT_CANCEL);
               Msg(EXIT_COMM,"comm","Program area already in use");
             } else {
               COMsysPutChar(CASPORT_OVERWRITE);
               if ((rdchar=COMsysGetChar())!=CASPORT_OK)
                 Msg(EXIT_COMM,"comm","Unable to overwrite program: 0x%02x",rdchar);
               Msg(M_DEBUG,"comm","Overwriting existing program");
             }
             break;
    case CASPORT_MEMFULL:
             Msg(EXIT_COMM,"comm","Calculator is out of free memory");
             break;
    case CASPORT_WRONGTYPE:
             Msg(EXIT_COMM,"comm","Wrong kind of data");
             break;
    case CASPORT_BADFILE:
             Msg(EXIT_COMM,"comm","Corrupted CAS file");
             break;
    default: Msg(M_WARN,"comm","Unknown response code: 0x%02x",response);
  }
}


void COMReadRaw(casinfo *info_r)
{
  unsigned int rdchar;
  int prev,success=TRUE;
  long size=0;
  int bsize=0;
  int i=0;
  unsigned char *data;

  data=info_r->data;

  for (;;) {
    prev=success;
    if (size>CAS_MEMSIZE)
      Msg(EXIT_FILE,"com/read","File too large: %ld",size);
    rdchar=COMsysGetCharTimeout(&success,100);
    if (!success) {
      if (!prev) {
        break;
      } else {
        COMsysPutChar(CASPORT_OK);
        if (flags.verbose) printf(" ok");
        if (flags.debug) printf(" (%u)",bsize);
        if (flags.verbose||flags.debug) putchar('\n');
        size+=bsize;bsize=0;
      }
    } else {
      data[i++]=rdchar;
      if (flags.verbose||flags.debug)
        if (bsize%CASPORT_DOTSIZE==0) {
          putchar('.');
          fflush(stdout);
        }
      bsize++;
    }
  }

  /*  --size;  */
  /* For some communications, there is a spurious character at the end, but
     sometimes there isn't.  Best let it lie... */
  /* Sometimes there is a whole spurious block */

  Msg(M_DEBUG,"comm/size","Data size: %d bytes",size);
  Msg(M_VERBOSE,"comm","Received data");

  info_r->type=tRaw;
  info_r->length=size;
  info_r->numblks=1;
}

void COMWriteRaw(casinfo *info)
{
  unsigned int rdchar;
  int bsize,i,success;
  unsigned char *data;

  if (flags.input.subf!=SUBF_RAW)
    Msg(EXIT_CMD,"comm/wr-raw","Cannot convert data to raw subfiletype");

  data=info->data;

  bsize=0;
  for (i=0;i<info->length;i++) {
    COMsysPutChar(data[i]);
    if (flags.verbose||flags.debug)
      if (bsize%CASPORT_DOTSIZE==0) {
        putchar('.');
        fflush(stdout);
      }
    bsize++;
    /* New blocks always start with a colon, however, not *every* colon
     * is the start of a new block, so we must have the timeout.
     */
    if (i+1==info->length || data[i+1]==':') {
      rdchar=COMsysGetCharTimeout(&success,100);
      if (success) {
        if (flags.verbose) printf(" ok");
        if (flags.debug) printf(" (%u)",bsize);
        if (flags.verbose||flags.debug) putchar('\n');
        bsize=0;
      }
    }
  }
  Msg(M_VERBOSE,"comm","Transmitted data");
  return;

}


#else /* CAS_COMM */

void COMReadData(char *filename, casinfo *info_r)
{
  Msg(EXIT_BUG,"com/read","Serial communications not yet supported on this platform");
}

void COMWriteData(char *filename,casinfo *info)
{
  Msg(EXIT_BUG,"com/write","Serial communications not yet supported on this platform");
}

#endif /* CAS_COMM */

/* end of fcom.c */
