/*      CaS, performs conversion from textfiles to .CAS files
 *
 * (c) 1996 Tom Wheeley <tomw@tsys.demon.co.uk>  All Rights Reserved
 *
 * You may freely use parts of this source code, provided that you:
 *  + Acknowledge the fact you used CaS source code in your program, visibly
 *    to your program's users.
 *  + Send the author of CaS a copy of your program.
 *  + You are not under obligation to include source for your program, but
 *    that is preferred.
 *
 */
 
#include "cas.h"
#include "misc.h"
#include "fcas.h"
#include "ffxp.h"

extern flagtype flags;

void FXPWriteData(char *filename, casinfo *info)
{
  FILE *fxp=NULL;
  prginfo *pi=NULL;
  int n;

  pi=info->aux;

  fxp=tsys_fopen(filename,"w",FALSE);
  fprintf(fxp,"%d\n%d\n",pi->size+2,':');
  for (n=0;n<pi->size;n++) {
    fprintf(fxp,"%d\n",pi->data[n]);
  }
  fprintf(fxp,"%d\n0\n",CalcCSum(pi->data,pi->size));

  tsys_fclose(fxp);
}


void FXPReadData(char *filename, casinfo *info_r)
{
  char line[CAS_FXP_LINELENGTH];
  FILE *fxp=NULL;
  prginfo *pi=NULL;
  unsigned char *data=NULL;
  int n;
  int rcsum,ccsum;

  pi=info_r->aux=tsys_malloc(sizeof (prginfo));
  data=pi->data=info_r->data;
  info_r->size1=1;
  info_r->type=tSingleProg;
  info_r->numblks=1;

  if (!(fxp=tsys_fopen(filename,"r",FALSE))) {
    Msg(EXIT_FILE,"fxp/read","Unable to open `%s'",filename);
    return;
  }

  fgets(line,CAS_FXP_LINELENGTH,fxp);  /* read size info */
  info_r->length=info_r->size2=atoi(line);
  pi->size=info_r->size2-2;

  fgets(line,CAS_FXP_LINELENGTH,fxp);  /* eat colon */

  for (n=0;n<pi->size;n++) {
    fgets(line,CAS_FXP_LINELENGTH,fxp);
    *(data++)=(unsigned) atoi(line) & 0xff;
  }

  fgets(line,CAS_FXP_LINELENGTH,fxp);
  rcsum=atoi(line);
  ccsum=CalcCSum(pi->data,pi->size);

  if (rcsum!=ccsum) {
    Msg(M_WARN,"fxp/csum","Checksum error.  read 0x%02x, expected 0x%02x",
                rcsum,ccsum);
  }

  tsys_fclose(fxp);
}


/* end of ffxp.c */
