/*      CaS, performs conversion from textfiles to .CAS files
 *
 * (c) 1996 Tom Wheeley <tomw@tsys.demon.co.uk>  All Rights Reserved
 *
 * You may freely use parts of this source code, provided that you:
 *  + Acknowledge the fact you used CaS source code in your program, visibly
 *    to your program's users.
 *  + Send the author of CaS a copy of your program.
 *  + You are not under obligation to include source for your program, but
 *    that is preferred.
 *
 */
 
#include "cas.h"
#include "misc.h"
#include "args.h"
#include "token.h"
#include "list.h"
#include "s9700.h"
#include "ftxt.h"

extern flagtype flags;

void TXTWriteData(char *filename,casinfo *info,tokenlist *toklis)
{
  FILE *txt=NULL;
  int ctoken;
  unsigned char *data=NULL;
  prginfo *pi=NULL;
  int n;

  static char *ProgramName[]={"0","1","2","3","4","5","6","7","8","9",
                              "A","B","C","D","E","F","G","H","I","J",
                              "K","L","M","N","O","P","Q","R","S","T",
                              "U","V","W","X","Y","Z","\\r","\\theta"};

  if (info->type!=tSingleProg && info->type!=tMultiProg &&
      info->type!=tSingleEditor && info->type!=tMultiEditor) {
    Msg(EXIT_BUG,"txt/write","Unsupported file variation: %s",ListType(info->type));
    return;
  }

  pi=info->aux;
  data=pi->data;

  txt=tsys_fopen(filename,"w",FALSE);
  fprintf(txt,"# Created by CaS %s\n",CAS_VER_STR);

  pi=info->aux;

  for (n=0;n<info->size1;n++) {
    if (pi[n].size>1) {

      switch(info->type) {
        case tSingleProg: fprintf(txt,"\n@@ Prog ( ");break;
        case tMultiProg: fprintf(txt,"\n@@ Prog %s ( ",ProgramName[n]);break;
        case tSingleEditor:; case tMultiEditor:
          fprintf(txt,"\n@@ Editor \"%s\"",pi[n].filename);
          if (ArgsIsSet("pass",flags.output.argv))
            fprintf(txt," (%s)",pi[n].password);
          break;
        default:;
      }

      if (info->type==tSingleProg || info->type==tMultiProg) {
        if (pi[n].progtype & CAS_PT_SD)     fprintf(txt,"SD ");
        if (pi[n].progtype & CAS_PT_LR)     fprintf(txt,"LR ");
        if (pi[n].progtype & CAS_PT_MATRIX) fprintf(txt,"MATRIX ");
        if (pi[n].progtype & CAS_PT_BASEN)  fprintf(txt,"BASE-N ");
        if (pi[n].progtype & CAS_PT_SSTO)   fprintf(txt,"SSTO ");
        if (pi[n].progtype & CAS_PT_SDRAW)  fprintf(txt,"SDRAW ");
        fprintf(txt,")\n\n");
      } else {
        fprintf(txt,"\n\n");
      }

      data=pi[n].data;
      for (;;) {
        ctoken=*(data++);
        if (ctoken==0x7f) {
          ctoken=256 + *(data++);
        }
        if (ctoken==0xf7) {
          ctoken=512 + *(data++);
        }
        if (ctoken==0) {
          Msg(M_DEBUG,"txt/write","Program terminated with 0");
          break;
        }
        if (ctoken==255) break;

        fprintf(txt,"%s",CodeToToken(ctoken,toklis,FALSE));
      }
      putc('\n',txt);
    }
  }

  tsys_fclose(txt);
}


void TXTReadData(char *filename,casinfo *info_r,tokenlist *toklis)
{
  FILE *txt=NULL;
  int ctoken;
  unsigned char *data;
  prginfo *pi=NULL;
  char line[PARSE_LINESIZE];
  char *lpos=NULL;
  int n,lineno;

  if (!(txt=tsys_fopen(filename,"r",FALSE))) {
    Msg(EXIT_FILE,"txt/read","Unable to open `%s'",filename);
    return;
  }

  info_r->aux=pi=tsys_calloc(CAS_PZ_NUMPROGS * sizeof(prginfo));
  info_r->type=tUnknown;
  info_r->size1=1;
  info_r->numblks=1;

  data=pi->data=info_r->data;
  n=0;lineno=0;

  for (;;) {
    *line='\0';
    while (!*line || *line=='\n' || *line=='#' || *line==';') {
      fgets(line,PARSE_LINESIZE,txt);lineno++;
      if (feof(txt)) goto finished;
      TXTStripComments(line,lineno);
    }
    lpos=line;

    if (!strncmp(line,"@@",2)) {

      if (pi[n].size>1) {
        if (*(data-1)==0x0d) {
          data--;pi[n].size--;
        }
        *(data++)=0xff;pi[n].size++;
      }

      TXTReadHeader(line,info_r, &n, lineno);

      switch (info_r->type) {
        case tSingleProg: break;
        case tSingleEditor: break;
        case tMultiProg:
            pi[n].data=data;
            break;
        case tMultiEditor:
            pi[n].data=data;
            break;

        default:
            Msg(EXIT_BUG,"txt/read","Unsupported file variation: %s",ListType(info_r->type));
            return;
      }

      *line='\0';
      continue;
    }

    for(;;) {
      if (!(ctoken=TXTReadToken(txt,toklis,&lpos,lineno)))
        break;

      if (ctoken < 256) {
        *(data++)=ctoken;
        pi[n].size++;
      } else {
        if (ctoken < 512) {
          *(data++)=0x7f;
          *(data++)=ctoken-256;
          pi[n].size+=2;
        } else {
          *(data++)=0xf7;
          *(data++)=ctoken-512;
          pi[n].size+=2;
        }
      }

      if ((void *)data >= info_r->dataend)
        Msg(EXIT_MEM,"txt/read","Filled data buffer");
    }
  }

  finished:

  if (pi[n].size>1) {
    if (*(data-1)==0x0d) {
      data--;pi[n].size--;
    }
    *(data++)=0xff;pi[n].size++;
  }

  tsys_fclose(txt);

  if (info_r->type==tUnknown) info_r->type=tSingleProg;

  info_r->size2=2;
  for (n=0;n<info_r->size1;n++) {
    if (pi[n].size<=1)
      info_r->size2++;
    else
      info_r->size2+=pi[n].size;
  }
  info_r->length=info_r->size2;
  if (info_r->type==tMultiEditor) info_r->numblks=info_r->size1;
}


bool TXTReadHeader(char *line, casinfo *info_r, int *n, int lineno)
{
  char *type=NULL;
  char *name=NULL;
  char *parm=NULL;
  char *tmp=NULL;
  prginfo *pi=NULL;

  if ((tmp=strrchr(line,'\n'))!=NULL)  /* remove terminating newline */
    *tmp='\0';

  /* Only called when beginning of line is "@@" */
  type = line + strspn(line,"@ ");    /* remove preceeding `@@ ' */

  if ((parm=strchr(line,'('))!=NULL) {
    *(parm++)='\0';
    if ((tmp=strrchr(parm,')'))!=NULL)
      *tmp='\0';
    else
      Msg(M_WARN,"txt/readhdr","%d: Unclosed parentheses",lineno);
  } else
    parm="";

  if ((name=strchr(type,' '))!=NULL) {
     *(name++)='\0';
     if ((tmp=strchr(name,' '))!=NULL)
       *tmp='\0';
  } else
    name="";

  if (!stricmp(type,"Prog") || !stricmp(type,"Program")) {
    pi=info_r->aux;
    if (!name||!*name) {    /* Single */
      *n=0;
      if (pi->size || info_r->type!=tUnknown) {
        Msg(EXIT_FORMAT,"txt/readhdr","%d: Single program header after program data",lineno);
      }
      info_r->type=tSingleProg;
    } else {        /* Multiple */
      if (info_r->type==tSingleProg) {
        Msg(EXIT_FORMAT,"txt/readhdr","%d: Single program header in multiple program file",lineno);
      }
      info_r->type=tMultiProg;
      info_r->size1=CAS_PZ_NUMPROGS;
      *n=-1;
      if (strlen(name)==1) {
        *n=strtol(name,&tmp,36);
        if (*tmp) *n=-1;
      } else {
        if (!stricmp(name,"\\r")) *n=36;
        if (!strnicmp(name,"\\th",3)) *n=37;
      }
      if (*n<0 || *n>37) {
        Msg(EXIT_FORMAT,"txt/readhdr","%d: Unknown program slot `%s'",lineno,name);
      }
    }

    pi[*n].progtype=0;
    if (parm && *parm) {
      for(tmp=strtok(parm," ");tmp;tmp=strtok(NULL," ")) {
        if (!stricmp(tmp,"SD"))     pi[*n].progtype |= CAS_PT_SD;
        if (!stricmp(tmp,"LR"))     pi[*n].progtype |= CAS_PT_LR;
        if (!stricmp(tmp,"MATRIX")) pi[*n].progtype |= CAS_PT_MATRIX;
        if (!stricmp(tmp,"BASE-N")) pi[*n].progtype |= CAS_PT_BASEN;
        if (!stricmp(tmp,"SSTO"))   pi[*n].progtype |= CAS_PT_SSTO;
        if (!stricmp(tmp,"SDRAW"))  pi[*n].progtype |= CAS_PT_SDRAW;
      }
    }

    return FALSE;
  }

  if (!stricmp(type,"Editor") || !stricmp(type,"Edit")) {
    pi=info_r->aux;
    if (info_r->type==tSingleEditor) info_r->type=tMultiEditor;
    if (info_r->type==tUnknown) info_r->type=tSingleEditor;
    if (info_r->type!=tMultiEditor && info_r->type!=tSingleEditor)
      Msg(EXIT_FORMAT,"txt/readhdr","%d: Editor file header after program data",lineno);
    if (info_r->type==tSingleEditor) {
      *n=0;
      info_r->size1=1;
    } else {
      info_r->size1++;
      ++*n;
    }
    if (*name=='\"' && name[strlen(name)-1]=='\"') {
      name++;
      name[strlen(name)-1]='\0';
    }
    strncpy((char *)pi[*n].filename,name,12);
    if (parm)
      strncpy((char *)pi[*n].password,parm,12);

    return FALSE;
  }

  if (!stricmp(type,"Matrix")) {
    return TRUE;
  }
  if (!stricmp(type,"Function")) {
    return TRUE;
  }
  if (!stricmp(type,"Vars")) {
    return TRUE;
  }
  if (!stricmp(type,"Matrix")) {
    return TRUE;
  }
  if (!stricmp(type,"StdDev")) {
    return TRUE;
  }
  if (!stricmp(type,"LinReg")) {
    return TRUE;
  }

  Msg(EXIT_FORMAT,"txt/readhdr","%d: Unknown file type `%s'",lineno,type);
  info_r->type=tUnknown;
  return TRUE;
}


int TXTReadToken(FILE *txt, tokenlist *toklis, char **lpos, int lineno)
{
  static char badtok[PARSE_LINESIZE];
  static enum {insOut,insComment,insString} instr=insOut;

  int ctok=0;
  char token[PARSE_TOKSIZE+1];
  char *tpos;

  /* Clearing up (called with NULL,NULL) */

  if (!lpos||!*lpos) {
    if (*badtok) {
      Msg(M_ERROR,"parse","%d: unknown token or part of token `%s'",lineno,badtok);
    }
    return 0;
  }

  if (!**lpos) return 0;

  /* remove leading spaces unless in text or comment */

  if (instr==insOut && (**lpos==' '||**lpos=='\t')) {
    while(**lpos==' '||**lpos=='\t')
      ++(*lpos);
  }

     /* a line ends with no \n */
  if (!**lpos) {
    Msg(M_DEBUG,"parse","%d: line ends on whitespace",lineno);
    return 0;
  }
  
     /* support for `\\' evaluating to `' to break up tokens */

  if (!strncmp(*lpos,"\\\\",2)) {
    (*lpos)+=2;
    return TXTReadToken(txt,toklis,lpos,lineno);
  }

  /* Start with TOKSIZE size token, and reduce until valid token */

  strncpy(token,*lpos,PARSE_TOKSIZE);
  token[PARSE_TOKSIZE]='\0';
  tpos=token+strlen(token);
  while(tpos!=token) {
    if ((ctok=TokenToCode(token,toklis,instr==insOut?FALSE:TRUE))!=0) break;
    *(--tpos)='\0';
  }

  /* If no token found */

  if (ctok==0) {
    strncat(badtok,*lpos,1); /* add token to list of bad tokens */
    (*lpos)++;
    return TXTReadToken(txt,toklis,lpos,lineno);
  } else if (*badtok) {
    Msg(M_ERROR,"parse","%d: unknown token or part of token `%s'",lineno,badtok);
    *badtok='\0';
  }

  /* Warning!  Hardcoded ctokens here! */
  if (ctok==34) {  /*  Double quotes  "  */
    switch (instr) {
      case insOut: instr=insString;break;
      case insString: instr=insOut;break;
      default:;
    }
  }
  if (ctok==39) {  /* Apostrophe (calc comment)  '  */
    instr=insComment;
  }
  if ((ctok==13 || ctok==58) && instr==insComment) instr=insOut; /* \n colon */
  
  (*lpos)+=(tpos-token);  
  return ctok;
}


void TXTStripComments(char *line, int lineno)
{
  char *tmp;
  bool instr=FALSE;

  if ((tmp=strrchr(line,'\n'))) *tmp='\0';      /* remove \n */

  for (tmp=line;*tmp;tmp++) {                   /* remove comments */
    if (*tmp=='"') instr=!instr;                /* but not in strings */
    if (*tmp==' ' && !instr)
      if (*(tmp+1)==';'||*(tmp+1)=='#')
        *tmp='\0';
  }

  for (tmp=line+strlen(line)-1;tmp>=line;tmp--) {
    if (!isspace(*tmp)) break;                  /* remove trailing spaces */
    *tmp='\0';
  }

  for(tmp=line;*tmp;tmp++)                      /* Add \n to end of line */
    ;
  *tmp='\n';*(tmp+1)='\0';
}

/* end of ftxt.c */
