/*      CaS, performs conversion from textfiles to .CAS files
 *
 * (c) 1996 Tom Wheeley <tomw@tsys.demon.co.uk>  All Rights Reserved
 *
 * You may freely use parts of this source code, provided that you:
 *  + Acknowledge the fact you used CaS source code in your program, visibly
 *    to your program's users.
 *  + Send the author of CaS a copy of your program.
 *  + You are not under obligation to include source for your program, but
 *    that is preferred.
 *
 */
 
#include "cas.h"
#include "misc.h"
#include "args.h"
#include "token.h"
#include "fcas.h"
#include "list.h"

extern flagtype flags;

void ListData(casinfo *info,tokenlist *toklis)
{
  int ctoken;
  unsigned char *data;
  int n;
  prginfo *pi;
  char numstyle='0';
  char *temp;

  static char *ProgramName[]={"0","1","2","3","4","5","6","7","8","9",
                              "A","B","C","D","E","F","G","H","I","J",
                              "K","L","M","N","O","P","Q","R","S","T",
                              "U","V","W","X","Y","Z","\\r","\\theta"};

  switch (info->type) {
    case tSSMono:;case tSSColour:
      ListScreenshot(info);
      return;
    case tSingleProg:;case tMultiProg:;case tSingleEditor:;case tMultiEditor:
      break;  
    default:
      Msg(EXIT_BUG,"list","Unsupported file variation");
      return;
  }

  if (ArgsIsSet("spc",flags.list.argv)||ArgsIsSet("space",flags.list.argv))
    numstyle='s';
  if (ArgsIsSet("dec",flags.list.argv))
    numstyle='d';
  if (ArgsIsSet("hex",flags.list.argv))
    numstyle='h';
  if (ArgsIsSet("oct",flags.list.argv))
    numstyle='o';
  if ((temp=ArgsGetString("num",flags.list.argv)))
    numstyle=tolower(*temp);

  pi=info->aux;

  if (info->type==tSingleProg) {
    putchar('\n');
    if (flags.pager) Pager(PAGER_ADD);
  }

  for (n=0;n<info->size1;n++) {
    if (pi[n].size>1) {
      if (info->type==tSingleEditor||info->type==tMultiEditor) {
        if (ArgsIsSet("pass",flags.list.argv)) {
           if (pi[n].password[0])
             printf("\n**** Password: [%s] ****",pi[n].password);
           if (flags.pager) Pager(PAGER_ADD);
        }
        ListSeparator((char *)pi[n].filename);
      }
      if (info->type==tMultiProg) {
        ListSeparator(ProgramName[n]);
      }
      data=pi[n].data;
      for (;;) {
        ctoken=*(data++);
        if (ctoken==0x7f) {
          ctoken=256 + *(data++);
        }
        if (ctoken==0xf7) {
          ctoken=512 + *(data++);
        }
        if (ctoken==0) {
          Msg(M_DEBUG,"list","Program terminated with 0");
          break;
        }
        if (ctoken==0xff) break;
        ListToken(ctoken,toklis,numstyle);
      }
      putchar('\n');
      if (flags.pager) Pager(PAGER_ADD);
    }
  }
}


void ListToken(int ctoken, tokenlist *toklis, char numstyle)
{
  char tok[TOKEN_MAX];

  strcpy(tok,CodeToToken(ctoken,toklis,TRUE));

  switch (numstyle) {
    case 's': printf(" %s",tok);break;
    case 'd': printf(" {%d} %s",ctoken,tok);break;
    case 'h': printf(" {0x%03x} %s",ctoken,tok);break;
    case 'o': printf(" {0%03o} %s",ctoken,tok);break;
    default:
      printf("%s",tok);
  }
  fflush(stdout);
  if (flags.pager && strchr(tok,'\n')) Pager(PAGER_ADD);
}


void ListSeparator(char *name)
{
  printf("\n");
  if (flags.pager) Pager(PAGER_ADD);
  printf("**** Program : [%s] ****\n\n",name);
  if (flags.pager) {
    Pager(PAGER_ADD);
    Pager(PAGER_ADD);
  }
}


void ListInfo(casinfo *ci)
{
  printf("Filetype: %s\n",ListType(ci->type));
  if (flags.pager) Pager(PAGER_ADD);
}


char *ListType(types type)
{
  switch(type) {
    case tSingleProg:   return "Single Program";
    case tMultiProg:    return "Set of Programs";
    case tSingleEditor: return "Single Editor File";
    case tMultiEditor:  return "Set of Editor Files";
    case tOneFMem:      return "Single Function";
    case tSixFMem:      return "Multiple Functions";
    case tGraphs:       return "Graphs";
    case tStdDev:       return "Standard Deviation data";
    case tLinReg:       return "Regression Data";
    case tSSMono:       return "Mono Screenshot";
    case tSSColour:     return "Colour Screenshot";
    case tAll:          return "Everything";
    case tDynaGraph:    return "Dynamic Graph";
    case tBackup:       return "Backup";
    case tPolyEq:       return "Polynomial Equations";
    case tSimEq:        return "Simultaneous Equations";
    case tRecTab:       return "Recursion Table";
    case tFuncTab:      return "Function Table";
    case tGrZoom:       return "Graph Zoom Factor";
    case tGrRange:      return "Graph Range";
    case tDefMem:       return "Defined Memories";
    case tAlphaMem:     return "Alphanumeric Memories";
    case tVarMem:       return "Variable Memories";
    case tMultiMat:     return "Set of Matrices";
    case tSingleMat:    return "Single Matrix";
    case tEnd:          return "End of File";
    case tRaw:          return "Raw CAS data";
    case tUnknown:
    default:            return "Unknown";
  }
}


void ListDump(casinfo *info,tokenlist *toklis)
{
  unsigned char *data=NULL;
  int n,p;
  prginfo *pi=NULL;

  if (info->type!=tSingleProg && info->type!=tMultiProg &&
      info->type!=tSingleEditor && info->type!=tMultiEditor) {
    Msg(EXIT_BUG,"dump","Unsupported file variation");
    return;
  }

  pi=info->aux;
  printf("Number of programs: %02d.  Total Size: 0x%04x\n",
          info->size1,info->size2);

  for (n=0;n<info->size1;n++) {
    printf("*** Prog %02d, Size=0x%04x, type=%02x, ptr=%p, csum=%02x, name [%s]\n",
    n,pi[n].size,pi[n].progtype,pi[n].data,CalcCSum(pi[n].data,pi[n].size),pi[n].filename);

    data=pi[n].data;
    for (p=0;p<pi[n].size;p++) {
      printf("%02x, ",*(data++));
    }
    printf("(%02x)\n",*data);
  }

  printf("casinfo = {\n");
  printf("  type=%s\n",ListType(info->type));
  printf("  data=%p\n",info->data);
  printf("  dataend=%p\n",info->dataend);
  printf("  numblks=%d\n",info->numblks);
  printf("  size1=%d\n",info->size1);
  printf("  size2=%d\n",info->size2);
  printf("  aux=%p\n",info->aux);
  printf("}\n");
}


void ListScreenshot(casinfo *info)
{
  int x,y;
  unsigned char (*data)[SS_XMAX][SS_YMAX];
  char *tmp=NULL;
  int top,bot;
  int flg_inv,flg_sng,flg_crop,flg_num;
  char topchar,botchar,fullchar,emptychar;

  flg_inv=ArgsIsSet("inverse",flags.list.argv);
  flg_crop=ArgsIsSet("crop",flags.list.argv);
  flg_sng=ArgsIsSet("single",flags.list.argv);
  flg_num=ArgsIsSet("num",flags.list.argv);

  topchar=SS_ASC_TOP;botchar=SS_ASC_BOTTOM;
  fullchar=SS_ASC_FULL;emptychar=SS_ASC_EMPTY;
  if (ArgsIsSet("block",flags.list.argv)) {
    topchar=SS_DOS_TOP;botchar=SS_DOS_BOTTOM;
    fullchar=SS_DOS_FULL;emptychar=SS_DOS_EMPTY;
  }

  if ((tmp=ArgsGetString("top",flags.list.argv))) {
    if (*tmp=='$')
      if (tmp[1]) topchar=tmp[1];
    else
      topchar=(char)ArgsGetLong("top",topchar,flags.list.argv);
  }
  if ((tmp=ArgsGetString("bottom",flags.list.argv))) {
    if (*tmp=='$')
      if (tmp[1]) botchar=tmp[1];
    else
      botchar=(char)ArgsGetLong("bottom",botchar,flags.list.argv);
  }
  if ((tmp=ArgsGetString("full",flags.list.argv))) {
    if (*tmp=='$')
      if (tmp[1]) fullchar=tmp[1];
    else
      fullchar=(char)ArgsGetLong("full",fullchar,flags.list.argv);
  }
  if ((tmp=ArgsGetString("empty",flags.list.argv))) {
    if (*tmp=='$')
      if (tmp[1]) emptychar=tmp[1];
    else
      emptychar=(char)ArgsGetLong("empty",emptychar,flags.list.argv);
  }

  data=info->data;

  for (y=63;y>=0;y-=(flg_sng?1:2)) {
    if (flg_num) printf("%3d",y+1);
    for (x=0;x<(flg_crop?flg_num?76:79:info->size1);x++) {
      top=(*data)[x][y];
      if (flg_sng) bot=top; else bot=(*data)[x][y-1];
      if (flg_inv) {top=1-top;bot=1-bot;}
      putchar(top?bot?fullchar:topchar:bot?botchar:emptychar);
    }
    putchar('\n');
    if (flags.pager) Pager(PAGER_ADD);
  }

}



/* end of list.c */
