#
# Makefile for CaS, by Tom Wheeley <tomw@tsys.demon.co.uk>
#
CC = gcc
CFLAGS = -O6 -ansi -pedantic -Wall -g

PROGS = cas casport tokcheck castrate

SRCS1 = misc.c args.c token.c list.c ssconv.c ftxt.c ffxp.c fbmp.c fgif.c fcas.c fcom.c s9700.c cunix.c main.c
OBJS1 = $(SRCS1:.c=.o)

SRCS2 = misc.c cunix.c casport.c
OBJS2 = $(SRCS2:.c=.o)

SRCS3 = misc.c token.c tokcheck.c
OBJS3 = misc.o tokenc.o tokcheck.o

SRCS4 = castrate.c misc.c token.c
OBJS4 = $(SRCS4:.c=.o)

OBJS = $(OBJS1) $(OBJS2) $(OBJS3) $(OBJS4)

LIBS =

# Rules

all: $(PROGS)

cas: $(OBJS1)
	$(CC) $(CFLAGS) -o CaS $(OBJS1) $(LIBS)

casport: $(OBJS2)
	$(CC) $(CFLAGS) -o casport $(OBJS2) $(LIBS)

tokcheck: $(OBJS3)
	$(CC) $(CFLAGS) -o tokcheck $(OBJS3)

castrate: $(OBJS4)
	$(CC) $(CFLAGS) -o castrate $(OBJS4)

clean:
	-if exist *.o del *.o

new: clean all

obj:	$(OBJS)

# File dependencies (sholud be nice and automatic...)

main.o: main.c cas.h misc.h args.h token.h list.h ftxt.h fcas.h \
               ffxp.h fbmp.h fgif.h fcom.h ssconv.h main.h
misc.o: misc.c cas.h misc.h
args.o: args.c cas.h misc.h token.h args.h
token.o: token.c cas.h misc.h token.h
list.o: list.c cas.h misc.h args.h token.h fcas.h list.h
ssconv.o: ssconv.c cas.h misc.h args.h token.h list.h ssconv.h
ffxp.o: ffxp.c cas.h misc.h fcas.h ffxp.h
ftxt.o: ftxt.c cas.h misc.h args.h token.h list.h s9700.h ftxt.h
fbmp.o: fbmp.c cas.h misc.h args.h token.h list.h fbmp.h
fgif.o: fgif.c cas.h misc.h args.h token.h list.h fgif.h
fcas.o: fcas.c cas.h misc.h args.h s9700.h fcas.h
fcom.o: fcom.c cas.h misc.h args.h s9700.h fcom.h commsys.h
s9700.o: s9700.c cas.h misc.h args.h list.h token.h s9700.h
s9x50.o: s9x50.c cas.h misc.h args.h list.h token.h s9x50.h
cunix.o: cunix.c cas.h misc.h commsys.h
cturboc.o: cturboc.c cas.h misc.h async.h commsys.h
tokcheck.o: tokcheck.c cas.h misc.h token.h tokcheck.h
castrate.o: castrate.c cas.h misc.h token.h castrate.h
casport.o: casport.c cas.h misc.h casport.h commsys.h

#requires special compilation
tokenc.o: token.c cas.h misc.h token.h
	$(CC) $(CFLAGS) -DCAS_TOKCHECK -c $< -o $@

