/*      CaS, performs conversion from textfiles to .CAS files
 *
 * (c) 1996 Tom Wheeley <tomw@tsys.demon.co.uk>  All Rights Reserved
 *
 * You may freely use parts of this source code, provided that you:
 *  + Acknowledge the fact you used CaS source code in your program, visibly
 *    to your program's users.
 *  + Send the author of CaS a copy of your program.
 *  + You are not under obligation to include source for your program, but
 *    that is preferred.
 *
 */

#ifndef _CAS_MISC_H_
#define _CAS_MISC_H_

/*  +ve values denote FATAL and program error code, see EXIT_* in cas.h  */

#define M_MSG           0
#define M_WARN          -1
#define M_ERROR         -2
#define M_VERBOSE       -3
#define M_DEBUG         -4

#define PAGER_LINES         20

#define PAGER_CLEAR         0       /* clears line count */
#define PAGER_FORCE         1       /* ignores flags.pager setting */
#define PAGER_ADD           2       /* Standard.  count++, page if >=20 */
#define PAGER_NOW           4       /* Page now, even if count!=20 */
#define PAGER_NOMSG         8       /* Don't display message */
#define PAGER_NOPAUSE       0x10    /* don't page, whatever (just incr) */
#define PAGER_NORESET       0x20    /* Don't reset counter to 0 on page */
#define PAGER_NOADD         0x40    /* don't increment counter */

#ifndef TRUE
#define TRUE    1           /* use only for assignment, never comparison */
#endif
#ifndef FALSE
#define FALSE    0
#endif

typedef signed char             Int8;
typedef unsigned char           UInt8;
typedef short int               Int16;
typedef unsigned short int      UInt16;
typedef long                    Int32;
typedef unsigned long           UInt32;
typedef float                   Float32;

#if defined(__linux__)
#define stricmp(x,y)            strcasecmp((x),(y))
#define strnicmp(x,y,z)         strncasecmp((x),(y),(z))
# ifdef __STRICT_ANSI__
int	strcasecmp(const char *_s1, const char *_s2);
int	strncasecmp(const char *_s1, const char *_s2, size_t _n);
# endif
#endif

#ifdef __DJGPP__    /* Not prototyped when -ansi, but they are in libc */
# ifdef __STRICT_ANSI__
int	stricmp(const char *_s1, const char *_s2);
int	strnicmp(const char *_s1, const char *_s2, size_t _n);
# endif
#endif

#ifndef FILENAME_MAX    /* probably hidden in some TurboC header file... */
#define FILENAME_MAX        127
#endif

/* prototypes */

void            Msg       (int type, const char *section, const char *msgstr, ...);
void            Pager     (bool inc);
unsigned long   Hash      (char *s, unsigned max);
int             WildMatch (char *wildcard, char *teststr);
unsigned char   CalcCSum  (void *data, int size);

/* wrappers for standard library functions */

void    *tsys_malloc      (size_t size);
void    *tsys_calloc      (size_t size);
void    *tsys_realloc     (void *oldptr, size_t size);
FILE    *tsys_fopen       (char *fname, char *faccess, int owndir);
int     tsys_fclose       (FILE *handle);


#ifdef FAT_ENDIAN
UInt32 SwapInt32	(UInt32 x);
#else
#define SwapInt32(x)	(x)
#endif /* FAT_ENDIAN */

#endif /* _CAS_MISC_H_ */

/* end of misc.h */
