/*      CaS, performs conversion from textfiles to .CAS files
 *
 * (c) 1996 Tom Wheeley <tomw@tsys.demon.co.uk>  All Rights Reserved
 *
 * You may freely use parts of this source code, provided that you:
 *  + Acknowledge the fact you used CaS source code in your program, visibly
 *    to your program's users.
 *  + Send the author of CaS a copy of your program.
 *  + You are not under obligation to include source for your program, but
 *    that is preferred.
 *
 */
 
#include "cas.h"
#include "misc.h"
#include "args.h"
#include "list.h"
#include "s9700.h"

extern flagtype flags;
extern unsigned char pow2[8];

static void RdInfoP1(casinfo *info_r,casheader *hdr,int curblk);
static void RdDataP1(casinfo *info_r,unsigned char *block,int curblk,void(* rdfunc)(unsigned char *,size_t));
static void WrInfoP1(casheader *hdr,casinfo *info,int curblk);
static void WrDataP1(casinfo *info,unsigned char *block,int curblk,void(* wrfunc)(unsigned char *,size_t));

static void RdInfoPZ(casinfo *info_r,casheader *hdr,int curblk);
static void RdDataPZ(casinfo *info_r,unsigned char *block,int curblk,void(* rdfunc)(unsigned char *,size_t));
static void WrInfoPZ(casheader *hdr,casinfo *info,int curblk);
static void WrDataPZ(casinfo *info,unsigned char *block,int curblk,void(* wrfunc)(unsigned char *,size_t));

static void RdInfoDD(casinfo *info_r,casheader *hdr,int curblk);
static void RdDataDD(casinfo *info_r,unsigned char *block,int curblk,void(* rdfunc)(unsigned char *,size_t));
static void WrInfoDD(casheader *hdr,casinfo *info,int curblk);
static void WrDataDD(casinfo *info,unsigned char *block,int curblk,void(* wrfunc)(unsigned char *,size_t));

static void RdInfoEN(casinfo *info_r,casheader *hdr,int curblk);
static void RdDataEN(casinfo *info_r,unsigned char *block,int curblk,void(* rdfunc)(unsigned char *,size_t));
static void WrInfoEN(casheader *hdr,casinfo *info,int curblk);
static void WrDataEN(casinfo *info,unsigned char *block,int curblk,void(* wrfunc)(unsigned char *,size_t));

static void RdInfoFN(casinfo *info_r,casheader *hdr,int curblk);
static void RdDataFN(casinfo *info_r,unsigned char *block,int curblk,void(* rdfunc)(unsigned char *,size_t));
static void WrInfoFN(casheader *hdr,casinfo *info,int curblk);
static void WrDataFN(casinfo *info,unsigned char *block,int curblk,void(* wrfunc)(unsigned char *,size_t));

/* not to be confused with castypes in other c*.c */

typedef struct {
  char magic[2];
  types type;
  int multi;
  void (*decodeinfo)(casinfo *,casheader *,int);
  void (*decodedata)(casinfo *,unsigned char *,int,void(*)(unsigned char *,size_t));
  void (*encodeinfo)(casheader *,casinfo *,int);
  void (*encodedata)(casinfo *,unsigned char *,int,void(*)(unsigned char *,size_t));
} castypes_t;

castypes_t castypes[] = {
  {"AA",tDynaGraph,FALSE,NULL,NULL,NULL,NULL},
  {"AD",tVarMem,FALSE,NULL,NULL,NULL,NULL},
  {"AL",tAll,FALSE,NULL,NULL,NULL,NULL},
  {"AM",tAlphaMem,FALSE,NULL,NULL,NULL,NULL},
  {"BU",tBackup,FALSE,NULL,NULL,NULL,NULL},
  {"DC",tSSColour,FALSE,NULL,NULL,NULL,NULL},
  {"DD",tSSMono,FALSE,RdInfoDD,RdDataDD,WrInfoDD,WrDataDD},
  {"DM",tDefMem,FALSE,NULL,NULL,NULL,NULL},
  {"EN",tSingleEditor,FALSE,RdInfoEN,RdDataEN,WrInfoEN,WrDataEN},
  {"FN",tMultiEditor,TRUE,RdInfoFN,RdDataFN,WrInfoFN,WrDataFN},
  {"F1",tOneFMem,FALSE,NULL,NULL,NULL,NULL},
  {"F6",tSixFMem,FALSE,NULL,NULL,NULL,NULL},
  {"GA",tGraphs,FALSE,NULL,NULL,NULL,NULL},
  {"GF",tGrZoom,FALSE,NULL,NULL,NULL,NULL},
  {"GR",tGrRange,FALSE,NULL,NULL,NULL,NULL},
  {"GT",tFuncTab,FALSE,NULL,NULL,NULL,NULL},
  {"MA",tMultiMat,FALSE,NULL,NULL,NULL,NULL},
  {"PD",tPolyEq,FALSE,NULL,NULL,NULL,NULL},
  {"P1",tSingleProg,FALSE,RdInfoP1,RdDataP1,WrInfoP1,WrDataP1},
  {"PZ",tMultiProg,FALSE,RdInfoPZ,RdDataPZ,WrInfoPZ,WrDataPZ},
  {"RT",tRecTab,FALSE,NULL,NULL,NULL,NULL},
  {"SD",tSimEq,FALSE,NULL,NULL,NULL,NULL},
  {"SR",tLinReg,FALSE,NULL,NULL,NULL,NULL},
  {"SS",tStdDev,FALSE,NULL,NULL,NULL,NULL},
  {"\x17\xff",tEnd,FALSE,NULL,NULL,NULL,NULL},
  {"::",tUnknown,FALSE,NULL,NULL,NULL,NULL}
};


void ReadData_9700(casinfo *info_r, void (* rdfunc)(unsigned char *, size_t))
{
  casheader *hdr;
  unsigned char *block=NULL;
  int match;
  int numblks;

  numblks=0;

  hdr=tsys_malloc(sizeof(casheader));
  block=tsys_malloc(CAS_MEMSIZE);
  info_r->type=tUnknown;
  
  do {
    (*rdfunc)((unsigned char *)hdr,sizeof(casheader));
    
    for (match=0;match<sizeof castypes/sizeof(castypes_t);match++)
      if (!strncmp(castypes[match].magic,(char *)hdr->magic,2)) break;

    if (castypes[match].type==tEnd) break; /* End of good file */

    if (castypes[match].type==tUnknown)
      Msg(EXIT_BUG,"9700/read","Unknown sub-block type `%c%c'",hdr->magic[0],hdr->magic[1]);

    Msg(M_DEBUG,"9700/read","filetype: %s",ListType(castypes[match].type));
    if (info_r->type==tUnknown) info_r->type=castypes[match].type;

    if (!castypes[match].decodeinfo || !castypes[match].decodedata)
      Msg(EXIT_BUG,"9700/read","Unsupported file-type: %s",ListType(castypes[match].type));

    (*castypes[match].decodeinfo)(info_r,hdr,numblks);
    (*castypes[match].decodedata)(info_r,block,numblks,rdfunc);
    numblks++;
  } while (castypes[match].multi);

  info_r->numblks=numblks;

  free(hdr);
  free(block);
}


void WriteData_9700(casinfo *info, void (* wrfunc)(unsigned char *, size_t))
{
  casheader *hdr;
  unsigned char *block=NULL;
  int match;
  int curblock;

  hdr=tsys_malloc(sizeof(casheader));
  block=tsys_malloc(CAS_MEMSIZE);

  for (match=0;match<sizeof castypes/sizeof(castypes_t);match++)
    if (castypes[match].type==info->type) break;

  Msg(M_DEBUG,"9700/write","filetype: %s",ListType(castypes[match].type));

  if (!castypes[match].encodeinfo || !castypes[match].encodedata)
    Msg(EXIT_BUG,"9700/write","Unsupported file variation: %s",ListType(castypes[match].type));

  for(curblock=0;curblock<info->numblks;curblock++) {
    ClearHeader(hdr,match,'\0');
    (*castypes[match].encodeinfo)(hdr,info,curblock);
    (*wrfunc)((unsigned char *)hdr,sizeof(casheader));
    (*castypes[match].encodedata)(info,block,curblock,wrfunc);
  }

  if (castypes[match].multi) {
    for (match=0;match<sizeof castypes/sizeof(castypes_t);match++) {
      if (castypes[match].type==tEnd) {
        ClearHeader(hdr,match,'\xff');
        (*wrfunc)((unsigned char *)hdr,sizeof(casheader));
        break;
      }
    }
  }

  free(hdr);
  free(block);
}


void ClearHeader(casheader *hdr, int matchnum, char misc)
{
  int i;

  for (i=0;i< 2;i++) hdr->magic[i]=castypes[matchnum].magic[i];
  for (i=0;i< 5;i++) hdr->misc[i]=misc;
  for (i=0;i<12;i++) hdr->filename[i]='\xff';
  for (i=0;i<12;i++) hdr->password[i]='\xff';
  for (i=0;i< 7;i++) hdr->reserved[i]='\xff';
}


static void RdInfoP1(casinfo *info_r,casheader *hdr,int curblk)
{
  prginfo *pi;

  info_r->aux=pi=tsys_calloc(sizeof(prginfo));
  info_r->size1 = 1;
  info_r->size2 = (hdr->misc[1] << 8)
                +  hdr->misc[2];
  pi->size=info_r->size2-2;
  pi->progtype=hdr->misc[3];
  pi->data=info_r->data;
}

static void RdDataP1(casinfo *info_r,unsigned char *block,int curblk,void(*rdfunc)(unsigned char *,size_t))
{
  (*rdfunc)(info_r->data,info_r->size2-2);
}

static void WrInfoP1(casheader *hdr,casinfo *info,int curblk)
{
  prginfo *pi;

  pi=info->aux;
  hdr->misc[1]=((pi->size+2) & 0xff00) >> 8;
  hdr->misc[2]=((pi->size+2) & 0x00ff);
  hdr->misc[3]=pi->progtype;
}

static void WrDataP1(casinfo *info,unsigned char *block,int curblk,void(* wrfunc)(unsigned char *,size_t))
{
  (*wrfunc)(info->data,info->size2-2);
}


static void RdInfoPZ(casinfo *info_r,casheader *hdr,int curblk)
{
  info_r->aux=tsys_calloc(CAS_PZ_NUMPROGS * sizeof(prginfo));
  info_r->size1 = CAS_PZ_NUMPROGS;
  info_r->size2 = (hdr->misc[1] << 8)
                +  hdr->misc[2];
}


static void RdDataPZ(casinfo *info_r,unsigned char *block,int curblk,void(*rdfunc)(unsigned char *,size_t))
{
  prginfo *pi;
  int n;
  unsigned char *curpos;
  prgheader ph[CAS_PZ_NUMPROGS];

  pi=info_r->aux;
  curpos=info_r->data;

  (*rdfunc)((unsigned char *)ph,sizeof ph);
  for (n=0;n<CAS_PZ_NUMPROGS;n++) {
    pi[n].size=(ph[n].sizehi << 8) + ph[n].sizelo;
    pi[n].progtype=ph[n].progtype;
    pi[n].data=curpos;
    curpos+=pi[n].size;
  }

  (*rdfunc)(info_r->data,info_r->size2-2);
}


static void WrInfoPZ(casheader *hdr,casinfo *info,int curblk)
{
  hdr->misc[1]=(info->size2 >> 8) & 255;
  hdr->misc[2]=(info->size2     ) & 255;
}


static void WrDataPZ(casinfo *info,unsigned char *block,int curblk,void(*wrfunc)(unsigned char *,size_t))
{
  int n;
  prgheader ph[CAS_PZ_NUMPROGS];
  prginfo *pi;
  unsigned char *curpos;

  pi=info->aux;
  curpos=block;

  for (n=0;n<CAS_PZ_NUMPROGS;n++) {
    ph[n].unknown1=0;ph[n].unknown3=0;
    if (pi[n].size==0) pi[n].size=1;
    ph[n].sizehi=(pi[n].size >> 8) & 255;
    ph[n].sizelo=(pi[n].size     ) & 255;
    ph[n].progtype=pi[n].progtype;

    if (pi[n].size==1) {
      *curpos++='\xff';
    } else {
      memcpy(curpos,pi[n].data,pi[n].size);
      curpos+=pi[n].size;
    }
  }

  (*wrfunc)((unsigned char *)ph,sizeof ph);
  (*wrfunc)(block,info->size2-2);
}


static void RdInfoDD(casinfo *info_r,casheader *hdr,int curblk)
{
  info_r->size1=hdr->misc[1];
  info_r->size2=hdr->misc[0];
  info_r->aux=tsys_malloc(sizeof(int));
  *(int *)(info_r->aux)=hdr->misc[3];
}

static void RdDataDD(casinfo *info_r,unsigned char *block,int curblk,void(* rdfunc)(unsigned char *,size_t))
{
  int x,y,z;
  unsigned char chunk;
  unsigned char *curpos;
  unsigned char (*data)[SS_XMAX][SS_YMAX];
  int flg_inv;

  flg_inv=ArgsIsSet("inverse",flags.input.argv);

  data=info_r->data;
  (*rdfunc)(block,(info_r->size1*info_r->size2)>>3);

  Msg(M_DEBUG,"cas/ss","%s style data",
    ((*(int *)(info_r->aux))==SS_9700_STYLE)?"9700":"9800");

  curpos=block;
  switch (*(int *)(info_r->aux)) {

    case SS_9700_STYLE:
      for (x=info_r->size1-1;x>0;x-=8)
        for (y=info_r->size2-1;y>=0;y--) {
          chunk=*curpos++;
          for (z=0;z<8;z++)
            (*data)[x-z][y]=(chunk & pow2[z])?!flg_inv:flg_inv;
        }
      break;

    case SS_9800_STYLE:
      for (y=info_r->size2-1;y>=0;y--)
        for (x=0;x<info_r->size1;x+=8) {
          chunk=*curpos++;
          for (z=0;z<8;z++)
            (*data)[x+z][y]=(chunk & pow2[7-z])?!flg_inv:flg_inv;
        }
      break;

    default: Msg(EXIT_FILE,"cas/ssread","Unknown data format");
  }
}

static void WrInfoDD(casheader *hdr,casinfo *info,int curblk)
{
  hdr->misc[0]=info->size2;    /* height */
  hdr->misc[1]=info->size1;    /* width */
  hdr->misc[2]=0;              /* unknown */
  hdr->misc[3]=SS_9700_STYLE;  /* data format */
  hdr->misc[4]='W';            /* unknown */
  hdr->filename[0]='F';        /* unknown */
}

static void WrDataDD(casinfo *info,unsigned char *block,int curblk,void(* wrfunc)(unsigned char *,size_t))
{
  int x,y,z;
  unsigned char (*data)[SS_XMAX][SS_YMAX];
  unsigned char chunk;
  unsigned char *curpos;
  int flg_inv;

  flg_inv=ArgsIsSet("inverse",flags.output.argv);

  data=info->data;
  curpos=block;

  for (x=info->size1-1;x>=0;x-=8)
    for (y=info->size2-1;y>=0;y--)
    {
      chunk=0;
      for (z=0;z<8;z++)
        chunk+=((*data)[x-z][y] ? !flg_inv:flg_inv) ? pow2[z] : 0;
      *curpos++=chunk;
    }
  (*wrfunc)(block,curpos-block);
}


static void RdInfoEN(casinfo *info_r,casheader *hdr,int curblk)
{
  prginfo *pi;

  info_r->aux=pi=tsys_calloc(sizeof(prginfo));
  info_r->size1 = 1;
  info_r->size2 = (hdr->misc[1] << 8)
                +  hdr->misc[2];
  pi->size=info_r->size2-2;
  pi->data=info_r->data;

  pi->filename[12]='\xff';
  strncpy((char *)pi->filename,(char *)hdr->filename,12);
  *(strchr((char *)pi->filename,'\xff'))='\0';

  pi->password[12]='\xff';
  strncpy((char *)pi->password,(char *)hdr->password,12);
  *(strchr((char *)pi->password,'\xff'))='\0';
}

static void RdDataEN(casinfo *info_r,unsigned char *block,int curblk,void(* rdfunc)(unsigned char *,size_t))
{
  (*rdfunc)(info_r->data,info_r->size2-2);
}

static void WrInfoEN(casheader *hdr,casinfo *info,int curblk)
{
  prginfo *pi;
  int i;

  pi=info->aux;
  hdr->misc[1]=((pi->size+2) & 0xff00) >> 8;
  hdr->misc[2]=((pi->size+2) & 0x00ff);

  strncpy((char *)hdr->filename,(char *)pi->filename,12);
  for (i=strlen((char *)pi->filename);i<12;i++)
    hdr->filename[i]='\xff';

  strncpy((char *)hdr->password,(char *)pi->password,12);
  for (i=strlen((char *)pi->password);i<12;i++)
    hdr->password[i]='\xff';
}

static void WrDataEN(casinfo *info,unsigned char *block,int curblk,void(* wrfunc)(unsigned char *,size_t))
{
  (*wrfunc)(info->data,info->size2-2);
}

static void RdInfoFN(casinfo *info_r,casheader *hdr,int curblk)
{
  prginfo *pi;

  info_r->aux=pi=tsys_realloc(info_r->aux,(curblk+1)*sizeof(prginfo));

  pi[curblk].size=((hdr->misc[1]) << 8)
                 +  hdr->misc[2] - 2;
  pi[curblk].data=info_r->data;
  if (curblk>0) pi[curblk].data=pi[curblk-1].data+pi[curblk-1].size;

  Msg(M_DEBUG,"9700/readFN","Editor file:(%d) %ld bytes, ptr=%p",curblk,pi[curblk].size,pi[curblk].data); 

  info_r->size1 = curblk+1;
  info_r->size2 += pi[curblk].size;

  pi[curblk].filename[12]='\xff';
  strncpy((char *)(pi[curblk].filename),(char *)(hdr->filename),12);
  *(strchr((char *)(pi[curblk].filename),'\xff'))='\0';

  pi[curblk].password[12]='\xff';
  strncpy((char *)(pi[curblk].password),(char *)(hdr->password),12);
  *(strchr((char *)(pi[curblk].password),'\xff'))='\0';
}

static void RdDataFN(casinfo *info_r,unsigned char *block,int curblk,void(* rdfunc)(unsigned char *,size_t))
{
  prginfo *pi;

  pi=info_r->aux;
  (*rdfunc)(pi[curblk].data,pi[curblk].size);
}

static void WrInfoFN(casheader *hdr,casinfo *info,int curblk)
{
  prginfo *pi;
  int i;

  pi=info->aux;
  hdr->misc[1]=((pi[curblk].size+2) & 0xff00) >> 8;
  hdr->misc[2]=((pi[curblk].size+2) & 0x00ff);

  strncpy((char *)hdr->filename,(char *)pi[curblk].filename,12);
  for (i=strlen((char *)pi[curblk].filename);i<12;i++)
    hdr->filename[i]='\xff';

  strncpy((char *)hdr->password,(char *)pi[curblk].password,12);
  for (i=strlen((char *)pi[curblk].password);i<12;i++)
    hdr->password[i]='\xff';

}

static void WrDataFN(casinfo *info,unsigned char *block,int curblk,void(* wrfunc)(unsigned char *,size_t))
{
  prginfo *pi;

  pi=info->aux;
  (*wrfunc)(pi[curblk].data,pi[curblk].size);
}

/* end of s9700.c */
