/*      tokcheck, checks CaS .tok token files.
 *
 * (c) 1996 Tom Wheeley <tomw@tsys.demon.co.uk>  All Rights Reserved
 *
 * You may freely use parts of this source code, provided that you:
 *  + Acknowledge the fact you used CaS source code in your program, visibly
 *    to your program's users.
 *  + Send the author of CaS a copy of your program.
 *  + You are not under obligation to include source for your program, but
 *    that is preferred.
 *
 */
 
#include "cas.h"
#include "misc.h"
#include "token.h"
#include "tokcheck.h"

flagtype flags={
  "",
  {FALSE,fUnknown,NULL},                /* list {set,format,num,tokfile} */
  {fUnknown,NULL,NULL,SUBF_UNKNOWN},    /* output {format,outfile,tokfile} */
  {fUnknown,NULL,NULL,SUBF_UNKNOWN},    /* input {set,format,infile,tokfile} */
  0,                                    /* conversions */
  FALSE,                                /* verbose */
  FALSE,                                /* pager */
  FALSE,                                /* memdump */
  NULL                                  /* debug {set,dbgf} */
};

int main(int argc, char **argv)
{
  FILE *mis;
  tokenlist *toklis;
  int h,i,o,l,nne;

  flags.argv0=argv[0];
  flags.debug=fopen(CAS_TOKC_DEBUG,"w");
  mis=fopen(CAS_TOKC_COUNT,"w");

  Msg(M_DEBUG,"tokcheck","Debug file for: %s",*argv);

  printf("tokcheck, (c) 1996 Tom Wheeley.  A part of CaS %s\n",CAS_VER_STR);
  if (argc<=1) {
    printf("\nusage: %s tokfile\n",*argv);
    exit(EXIT_CMD);
  }

  putchar('\n');
  ++argv;

  toklis=tsys_calloc(sizeof *toklis);
  Msg(M_DEBUG,"token","allocated %ld bytes",(long) sizeof *toklis);
  
  /* LoadControl(toklis); */
  LoadTokens(*argv,toklis,1,1,1);

  i=0;
  for (h=0;h<TOKEN_HASHSIZE;h++)
    if (!toklis->in[h].token) ++i;
  Msg(M_DEBUG,"hash","Of 0x%04x entries, 0x%04x used, 0x%04x empty",
                     TOKEN_HASHSIZE,TOKEN_HASHSIZE-i,i);

  if (mis) {
    int ctoken;
    fprintf(mis,"Token count file for: %s\n",*argv);
    fprintf(mis,"ctoken   : in,out,display.\n");
    printf("Counting tokens:");

    nne=0;
    for(ctoken=0;ctoken<TOKEN_MAX;ctoken++) {
      i=0;o=0;l=0;
      for (h=0;h<TOKEN_HASHSIZE;h++)
        if (toklis->in[h].code==ctoken) ++i;
      if (toklis->out[ctoken]) ++o;
      if (toklis->list[ctoken]) ++l;

      if ((int)(ctoken/CAS_TOKC_DOTINT)*CAS_TOKC_DOTINT==ctoken) {
        putchar('.');
        fflush(stdout);
      }
      fprintf(mis,"%4d,%4d: %2d,%2d,%2d.\n",ctoken,ctoken%256,i,o,l);
    }
    printf("*\nctoken count information written to %s\n",CAS_TOKC_COUNT);
    fclose(mis);
    Msg(M_DEBUG,"nne","read %d Not Normally Enterable tokens",nne);
    }

  if (flags.debug) {
    printf("Debug information written to %s\n",CAS_TOKC_DEBUG);
    fclose(flags.debug);
  }

  FreeTokens(toklis);

  return EXIT_SUCCESS;
}

/* end of tokcheck.c */
