/*      CaS, performs conversion from textfiles to .CAS files
 *
 * (c) 1996 Tom Wheeley <tomw@tsys.demon.co.uk>  All Rights Reserved
 *
 * You may freely use parts of this source code, provided that you:
 *  + Acknowledge the fact you used CaS source code in your program, visibly
 *    to your program's users.
 *  + Send the author of CaS a copy of your program.
 *  + You are not under obligation to include source for your program, but
 *    that is preferred.
 *
 */
 
#ifndef _CAS_TOKEN_H_
#define _CAS_TOKEN_H_

#define TOKEN_TOKFILE_ASC               "ascii.tok"
#define TOKEN_TOKFILE_DOS               "cp437.tok"

#define TOKEN_TOKSIZE                   255     /* Max line length in tok */
#define TOKEN_MIN                       1       /* Minimum ctoken number  */
#define TOKEN_MAX                       767     /* Number of ctokens      */
#define TOKEN_HASHSIZE                  0x2000  /* Size of hash table     */
#define TOKEN_BLOCKSIZE                 65500   /* size of token block    */

/*
 * The hash table must be large enough to contain every single pointer.
 * Extended clashes should also be avoided, so bigger == better
 * Inifinite loop will result if table too small!  Entries are (short+char*)
 */

#define TF_UNDEFINED            0 /* reads no tokens */
/* flags 0x01 -> 0x80 reserved for models, default allow all */
#define TF_DEFAULT              0x00ffU
#define TF_LISTONLY             0x0100U   /* Only used in list mode */
/* Following flags are stored */
#define TF_NNE                  0x4000U   /* not normally enterable */
#define TF_VERBATIM             0x8000U   /* not in strings */

/* Both flags and token number are stored in the `code' entry of the hash
   table, for space efficiency.  The flags are ANDed with the FLAG_MASK
   so that low-numbered flags (such as model info) doesn't interfere with
   token number information. */

#define TOKEN_MASK              0x03ffU
#define FLAG_MASK               0xfc00U

typedef struct {
  unsigned short code;              /* must be >= 16 bit */
  char *token;
} ht_entry;

typedef struct {
  ht_entry in[TOKEN_HASHSIZE];      /* hash table */
  char *out[TOKEN_MAX];             /* conventional table */
  char *list[TOKEN_MAX];            /* conventional table */
  char *tokblock;                   /* start of token text image */
  char *tokpos;                     /* current position in token image */
  char *tokend;                     /* end of token image (safety) */
} tokenlist;

tokenlist *InitTokens(void);
void LoadTokens(char *tokfile,tokenlist *toklis,int in,int out,int list);
int AddTokenLine(char *tokline,tokenlist *toklis,int in,int out,int list);
int TokenFlag(char **token_r);
char *DupToken(char *token, tokenlist *toklis);
unsigned long AddToHash(char *token,int toknum,tokenlist *toklis);
void DumpTokens(tokenlist *toklis);
void FreeTokens(tokenlist *toklis);
char *CodeToToken(int ctoken,tokenlist *toklis,bool display);
int TokenToCode(char *token,tokenlist *toklis, int verbatim);

#endif /* _CAS_TOKEN_H_ */

/* end of token.h */
